@@ -4,12 +4,15 @@ Based on the work of https://github.com/petretiandrea/plugp100
 under compatible GNU GPL3 license.
 """
 
+from __future__ import annotations
+
 import base64
 import hashlib
 import logging
 import time
+from collections.abc import AsyncGenerator
 from enum import Enum, auto
-from typing import TYPE_CHECKING, Any, AsyncGenerator, Dict, Optional, Tuple, cast
+from typing import TYPE_CHECKING, Any, Dict, cast
 
 from cryptography.hazmat.primitives import padding, serialization
 from cryptography.hazmat.primitives.asymmetric import padding as asymmetric_padding
@@ -22,12 +25,13 @@ from .deviceconfig import DeviceConfig
 from .exceptions import (
     SMART_AUTHENTICATION_ERRORS,
     SMART_RETRYABLE_ERRORS,
-    SMART_TIMEOUT_ERRORS,
-    AuthenticationException,
-    RetryableException,
-    SmartDeviceException,
+    AuthenticationError,
+    DeviceError,
+    KasaException,
     SmartErrorCode,
-    TimeoutException,
+    TimeoutError,
+    _ConnectionError,
+    _RetryableError,
 )
 from .httpclient import HttpClient
 from .json import dumps as json_dumps
@@ -91,19 +95,19 @@ class AesTransport(BaseTransport):
             self._login_params = json_loads(
                 base64.b64decode(self._credentials_hash.encode()).decode()  # type: ignore[union-attr]
             )
-        self._default_credentials: Optional[Credentials] = None
+        self._default_credentials: Credentials | None = None
         self._http_client: HttpClient = HttpClient(config)
 
         self._state = TransportState.HANDSHAKE_REQUIRED
 
-        self._encryption_session: Optional[AesEncyptionSession] = None
-        self._session_expire_at: Optional[float] = None
+        self._encryption_session: AesEncyptionSession | None = None
+        self._session_expire_at: float | None = None
 
-        self._session_cookie: Optional[Dict[str, str]] = None
+        self._session_cookie: dict[str, str] | None = None
 
-        self._key_pair: Optional[KeyPair] = None
-        self._app_url = URL(f"http://{self._host}/app")
-        self._token_url: Optional[URL] = None
+        self._key_pair: KeyPair | None = None
+        self._app_url = URL(f"http://{self._host}:{self._port}/app")
+        self._token_url: URL | None = None
 
         _LOGGER.debug("Created AES transport for %s", self._host)
 
@@ -117,14 +121,14 @@ class AesTransport(BaseTransport):
         """The hashed credentials used by the transport."""
         return base64.b64encode(json_dumps(self._login_params).encode()).decode()
 
-    def _get_login_params(self, credentials: Credentials) -> Dict[str, str]:
+    def _get_login_params(self, credentials: Credentials) -> dict[str, str]:
         """Get the login parameters based on the login_version."""
         un, pw = self.hash_credentials(self._login_version == 2, credentials)
         password_field_name = "password2" if self._login_version == 2 else "password"
         return {password_field_name: pw, "username": un}
 
     @staticmethod
-    def hash_credentials(login_v2: bool, credentials: Credentials) -> Tuple[str, str]:
+    def hash_credentials(login_v2: bool, credentials: Credentials) -> tuple[str, str]:
         """Hash the credentials."""
         un = base64.b64encode(_sha1(credentials.username.encode()).encode()).decode()
         if login_v2:
@@ -140,16 +144,14 @@ class AesTransport(BaseTransport):
         if error_code == SmartErrorCode.SUCCESS:
             return
         msg = f"{msg}: {self._host}: {error_code.name}({error_code.value})"
-        if error_code in SMART_TIMEOUT_ERRORS:
-            raise TimeoutException(msg, error_code=error_code)
         if error_code in SMART_RETRYABLE_ERRORS:
-            raise RetryableException(msg, error_code=error_code)
+            raise _RetryableError(msg, error_code=error_code)
         if error_code in SMART_AUTHENTICATION_ERRORS:
             self._state = TransportState.HANDSHAKE_REQUIRED
-            raise AuthenticationException(msg, error_code=error_code)
-        raise SmartDeviceException(msg, error_code=error_code)
+            raise AuthenticationError(msg, error_code=error_code)
+        raise DeviceError(msg, error_code=error_code)
 
-    async def send_secure_passthrough(self, request: str) -> Dict[str, Any]:
+    async def send_secure_passthrough(self, request: str) -> dict[str, Any]:
         """Send encrypted message as passthrough."""
         if self._state is TransportState.ESTABLISHED and self._token_url:
             url = self._token_url
@@ -170,7 +172,7 @@ class AesTransport(BaseTransport):
         # _LOGGER.debug(f"secure_passthrough response is {status_code}: {resp_dict}")
 
         if status_code != 200:
-            raise SmartDeviceException(
+            raise KasaException(
                 f"{self._host} responded with an unexpected "
                 + f"status code {status_code} to passthrough"
             )
@@ -184,14 +186,30 @@ class AesTransport(BaseTransport):
             assert self._encryption_session is not None
 
         raw_response: str = resp_dict["result"]["response"]
-        response = self._encryption_session.decrypt(raw_response.encode())
-        return json_loads(response)  # type: ignore[return-value]
+
+        try:
+            response = self._encryption_session.decrypt(raw_response.encode())
+            ret_val = json_loads(response)
+        except Exception as ex:
+            try:
+                ret_val = json_loads(raw_response)
+                _LOGGER.debug(
+                    "Received unencrypted response over secure passthrough from %s",
+                    self._host,
+                )
+            except Exception:
+                raise KasaException(
+                    f"Unable to decrypt response from {self._host}, "
+                    + f"error: {ex}, response: {raw_response}",
+                    ex,
+                ) from ex
+        return ret_val  # type: ignore[return-value]
 
     async def perform_login(self):
         """Login to the device."""
         try:
             await self.try_login(self._login_params)
-        except AuthenticationException as aex:
+        except AuthenticationError as aex:
             try:
                 if aex.error_code is not SmartErrorCode.LOGIN_ERROR:
                     raise aex
@@ -205,16 +223,16 @@ class AesTransport(BaseTransport):
                     "%s: logged in with default credentials",
                     self._host,
                 )
-            except AuthenticationException:
+            except (AuthenticationError, _ConnectionError, TimeoutError):
                 raise
             except Exception as ex:
-                raise AuthenticationException(
+                raise KasaException(
                     "Unable to login and trying default "
-                    + "login raised another exception: %s",
+                    + f"login raised another exception: {ex}",
                     ex,
                 ) from ex
 
-    async def try_login(self, login_params: Dict[str, Any]) -> None:
+    async def try_login(self, login_params: dict[str, Any]) -> None:
         """Try to login with supplied login_params."""
         login_request = {
             "method": "login_device",
@@ -257,7 +275,6 @@ class AesTransport(BaseTransport):
         self._session_expire_at = None
         self._session_cookie = None
 
-        url = f"http://{self._host}/app"
         # Device needs the content length or it will response with 500
         headers = {
             **self.COMMON_HEADERS,
@@ -266,7 +283,7 @@ class AesTransport(BaseTransport):
         http_client = self._http_client
 
         status_code, resp_dict = await http_client.post(
-            url,
+            self._app_url,
             json=self._generate_key_pair_payload(),
             headers=headers,
             cookies_dict=self._session_cookie,
@@ -275,7 +292,7 @@ class AesTransport(BaseTransport):
         _LOGGER.debug("Device responded with: %s", resp_dict)
 
         if status_code != 200:
-            raise SmartDeviceException(
+            raise KasaException(
                 f"{self._host} responded with an unexpected "
                 + f"status code {status_code} to handshake"
             )
@@ -318,7 +335,7 @@ class AesTransport(BaseTransport):
             or self._session_expire_at - time.time() <= 0
         )
 
-    async def send(self, request: str) -> Dict[str, Any]:
+    async def send(self, request: str) -> dict[str, Any]:
         """Send the request."""
         if (
             self._state is TransportState.HANDSHAKE_REQUIRED
@@ -330,7 +347,7 @@ class AesTransport(BaseTransport):
                 await self.perform_login()
             # After a login failure handshake needs to
             # be redone or a 9999 error is received.
-            except AuthenticationException as ex:
+            except AuthenticationError as ex:
                 self._state = TransportState.HANDSHAKE_REQUIRED
                 raise ex
 
@@ -355,7 +372,10 @@ class AesEncyptionSession:
         handshake_key_bytes: bytes = base64.b64decode(handshake_key.encode("UTF-8"))
         private_key_data = base64.b64decode(keypair.get_private_key().encode("UTF-8"))
 
-        private_key = serialization.load_der_private_key(private_key_data, None, None)
+        private_key = cast(
+            rsa.RSAPrivateKey,
+            serialization.load_der_private_key(private_key_data, None, None),
+        )
         key_and_iv = private_key.decrypt(
             handshake_key_bytes, asymmetric_padding.PKCS1v15()
         )
